<?php

/**
 * A class of common methods for custom citation styles and filter.
 *
 * @author Benjamin Sommer
 * @package netblog
 * @version 1.5
 * @since 1.5
 * @uses Wordpress 2.8+
 */
class nbcstyle {
	
	/**
	 * Add a custom filter for a custom citation style/
	 * 
	 * @uses get_option, add_option, update_option
	 * 
	 * @param string $style A custom citation style
	 * @param string $type A custom filter type
	 * @param string $cmd A string with attributes to format the output of the filter, without $.
	 * @return boolean TRUE on success, FALSE on failure (style is protected, style/type name too long, cmd too short)
	 */
	static public function addFilter( $style, $type, $cmd )
	{
		$default = self::getDftStyles();
		$cmdP = self::parseCMD($cmd);
		if( isset( $default[ strtolower($style) ] ) ) return false;
		
		if( strlen($style) > 20 || strlen($type) > 20 ) return false;
		if( strlen($style) < 3 || strlen($type) < 3 ) return false;
		if( strlen($cmdP) < 3 ) return false;
		
		$nm = self::mkFilterName($style,$type);
		if( get_option($nm) === false ) {
			add_option($nm,$cmdP,'','no');
			return true;
		} else { update_option($nm,$cmdP); return true; }		
	}
		
	
	/**
	 * Remove a custom filter
	 *
	 * @uses get_option, delete_option
	 * 
	 * @param string $style A custom style
	 * @param string $type A custom filter name to be removed
	 * @return boolean TRUE on success, FALSE on failure (removal failed, type not found)
	 */
	static public function rmFilter( $style, $type )
	{
		$nm = self::mkFilterName($style,$type);
		return get_option($nm) != false ? delete_option($nm) : false;
	}
	
	
	/**
	 * Get a custom filter command, as safe php-parsable code for eval().
	 *
	 * @uses get_option
	 * 
	 * @param string $style A custom style name
	 * @param string $type A custom type name for this style
	 * @return string The safe php-parsable filter command
	 */
	static public function getFilterCMD( $style, $type )
	{
		return get_option( self::mkFilterName($style,$type) );
	}
	
	
	/**
	 * Get all custom filter in an array, with filter_name => filter_cmd
	 *
	 * @uses wpdb, wpdb->get_results, wpdb->options
	 * 
	 * @param bool[optional] $formatted
	 * @return array An array of all custom filter, with option_name => option_value
	 */
	static public function getFilterRaw( $formatted = false )
	{
		global $wpdb;
		$out = array();
		$query = "
			SELECT o.option_name, o.option_value FROM $wpdb->options o
			WHERE o.option_name LIKE 'nbcite-%'
			ORDER BY o.option_name
		";
		$r = $wpdb->get_results($query,ARRAY_A);
		if($r!=null)	
		foreach( $r as $option ) {
			if($formatted) {
				$filter = $option['option_value'];
				$dft = self::getDftAtts();
				foreach( $dft as $k=>$v )
					$dft[$k] = strtoupper($k);//ucfirst($k);
				extract($dft);
				$option['option_value'] = eval("return \"$filter\" ;");
			}
			$out[ $option['option_name'] ] = $option['option_value'];
		}
		return $out;		
	}
	
	
	/**
	 * Get all filter for a given citation style, with type => cmd.
	 *
	 * @param string $style
	 * @param bool[optional] $formatted
	 * @return array
	 */
	static public function getFilter( $style, $formatted = false )
	{
		$out = array();
		$all = self::getFilterRaw($formatted);
		foreach($all as $nm=>$cmd) {
			$s = self::extractStyle($nm);			
			$t = self::extractType($nm);
			if( $s==$style ) {
				$out[$t] = $cmd;
			}
		}
		return $out;
	}
	
	
	/**
	 * Get a string of custom style names, separated by ,.
	 *
	 * @param string $output Any of STRING | ARRAY_N
	 * @return string A string of custom style name, separated by ','.
	 */
	static public function getStyles( $output = STRING )
	{
		$s = array();
		$r = self::getFilterRaw();
		foreach( $r as $k=>$v ) {
			$s[ self::extractStyle($k) ] =  self::extractStyle($k);
		}
		if($output == ARRAY_N ) {
			$t = array();
			foreach($s as $v) $t[] = $v;
			return $t;
		} else return implode(', ',$s);
	}
	
	
	/**
	 * Get an array of default/built-in citation styles, with short name => nice name.
	 *
	 * @return array An array of default styles
	 */
	static public function getDftStyles()
	{
		$t = array('apa'=>'APA','turabian'=>'Turabian','mla'=>'MLA','chicago'=>'Chicago');
		
		// MATCH nbcs_*.php
		$d = scandir( dirname(__FILE__) );
		foreach($d as $k=>$f ) {
			if( substr($f,0, 5) == 'nbcs_' && substr($f,-4) == '.php' && !isset( $t[$s=strtolower(substr($f,5,-4))] ) && $s != 'custom' ) {		
				$t[$s] = ucfirst($s);
			}
		}
		return $t;
	}
	
	
	/**
	 * Check if name is a valid style name.
	 *
	 * @param string $name
	 * @param string[optional] $type Get type of stylename
	 * @param string[optional] $classname
	 * @return bool
	 */
	static public function is_style( $name, &$type = null, &$classname )
	{
		if( self::isCustomStyle($name) ) {
			if($type!==null) $type = 'custom'; 
			if($classname!==null && file_exists($path=dirname(__FILE__)."/nbcs_custom.php") )
				 $classname = 'nbcs_custom';
			return true;
		}
		if( self::isDftStyle($name) ) {
			if($type!==null) $type = 'default';
			$name = strtolower($name);
			if($classname!==null && file_exists($path=dirname(__FILE__)."/nbcs_$name.php") ) 
				$classname = "nbcs_$name";
				
			return true;
		}
		return false;
	}
	
	
	/**
	 * Get an array of default attributes, with attr => default value
	 *
	 * @return array An array of default attributes
	 */
	static public function getDftAtts()
	{
		return array(
		'author' => '',
		'title' => '',
		'year' => '',
		'city' => '',
		'publisher' => '',
		'publisher_place' => '',
		'pages' => '',	
		'volume' => '',
		'issue' => '',
		'bookauthor' => '',
		'booktitle' => '',
		'title_periodical' => '',
		'initials' => '',
		'month' => '',
		'day' => '',
		'yearacc' => '',
		'monthacc' => '',
		'dayacc' => '',
		'url' => '',
		'doi' => '',
		'conference_name' => '',
		'award' => '',
		'yearorg' => '',
		'pagesorg' => '',
		'composer' => '',
		'conductor' => '',
		'performer' => '',
		'writer' => '',
		'theater' => '',
		'director' => '',
		'movie' => '',
		'inventor' => '',
		'patentnum' => '',
		'casenum' => '',
		'court' => '',
		'source' => '',
		'special' => '',
		'webpage' => '',			//=title
		'website' => '',			//=publisher
		'type' => '',
		'print' => '',
		'print_sections' => 'false',
		'print_headline' => '',
		'print_custom' => ''
      );
	}
	
	
	/**
	 * Get an array of default attributes with their name
	 *
	 * @return array An Array of default attributes, with attr => name
	 */
	static public function getDftAttsNamed()
	{
		return array(
			'author' => __('Author','netblog'),
			'title' => __('Title','netblog'),
			'year' => __('Year','netblog'),
			'city' => __('City','netblog'),
			'publisher' => __('Publisher','netblog'),
			'publisher_place' => __('Publisher Place','netblog'),
			'pages' => __('Pages','netblog'),	
			'volume' => __('Volume','netblog'),
			'issue' => __('Issue or Number','netblog'),			
			'bookauthor' => __('Book Author','netblog'),
			'booktitle' => __('Book Title','netblog'),
			'title_periodical' => __('Title Periodical','netblog'),
			'initials' => __('Author\'s Initials','netblog'),			
			'month' => __('Month','netblog'),
			'day' => __('Day','netblog'),
			'yearacc' => __('Year accessed','netblog'),
			'monthacc' => __('Month accessed','netblog'),
			'dayacc' => __('Day accessed','netblog'),
			'url' => __('URL','netblog'),
			'doi' => __('DOI','netblog'),
			'conference_name' => __('Conference name','netblog'),
			'award' => __('Award, e.g. PhD thesis','netblog'),
			'yearorg' => __('Books','netblog'),
			'pagesorg' => __('Books','netblog'),
			'composer' => __('Composer','netblog'),
			'conductor' => __('Conductor','netblog'),
			'performer' => __('Performer','netblog'),
			'writer' => __('Writer','netblog'),
			'theater' => __('Theater','netblog'),
			'director' => __('Director','netblog'),
			'movie' => __('Movie','netblog'),
			'inventor' => __('Inventor','netblog'),
			'patentnum' => __('Patent num.','netblog'),
			'casenum' => __('Case num.','netblog'),
			'court' => __('Court','netblog'),
			'source' => __('Source','netblog'),
			'special' => __('Special','netblog'),
			'webpage' => __('Webpage','netblog'),			//=title
			'website' => __('Website','netblog'),			//=publisher
			'type' => __('Type','netblog'),
			'print' => __('Print','netblog'),
			'print_sections' => __('Print Sections','netblog'),
			'print_headline' => __('Print Headline','netblog'),
			'print_custom' => __('Print Custom','netblog')
     	 );
	}
	
	
	/**
	 * Get an array of default atts with their help.
	 *
	 * @return array
	 */
	static public function getDftAttsHelp()
	{
		return array(
			'author' => '',
			'title' => '',
			'year' => __('e.g. 2010','netblog'),
			'city' => __('e.g. New York','netblog'),
			'publisher' => __('Publisher','netblog'),
			'publisher_place' => __('Publisher Place','netblog'),
			'pages' => __('Pages','netblog'),	
			'volume' => __('Volume','netblog'),
			'issue' => __('Issue or Number','netblog'),			
			'bookauthor' => __('Book Author','netblog'),
			'booktitle' => __('Book Title','netblog'),
			'title_periodical' => __('Title Periodical','netblog'),
			'initials' => __('Author\'s Initials','netblog'),			
			'month' => __('Month','netblog'),
			'day' => __('Day','netblog'),
			'yearacc' => __('Year accessed','netblog'),
			'monthacc' => __('Month accessed','netblog'),
			'dayacc' => __('Day accessed','netblog'),
			'url' => __('URL','netblog'),
			'doi' => __('DOI','netblog'),
			'conference_name' => __('Conference name','netblog'),
			'award' => __('Award, e.g. PhD thesis','netblog'),
			'yearorg' => __('Books','netblog'),
			'pagesorg' => __('Books','netblog'),
			'composer' => __('Composer','netblog'),
			'conductor' => __('Conductor','netblog'),
			'performer' => __('Performer','netblog'),
			'writer' => __('Writer','netblog'),
			'theater' => __('Theater','netblog'),
			'director' => __('Director','netblog'),
			'movie' => __('Movie','netblog'),
			'inventor' => __('Inventor','netblog'),
			'patentnum' => __('Patent num.','netblog'),
			'casenum' => __('Case num.','netblog'),
			'court' => __('Court','netblog'),
			'source' => __('Source','netblog'),
			'special' => __('Special','netblog'),
			'webpage' => __('Webpage','netblog'),			//=title
			'website' => __('Website','netblog'),			//=publisher
			'type' => __('Type','netblog'),
			'print' => __('Print','netblog'),
			'print_sections' => __('Print Sections','netblog'),
			'print_headline' => __('Print Headline','netblog'),
			'print_custom' => __('Print Custom','netblog')
     	 );
	}
	
	
	/**
	 * Get an array of default types of default citation styles
	 *
	 * @return array Array of default types, with type => name
	 */
	static public function getDftTypes()
	{
		return array(
			'book' => __('Books','netblog'),
			'booksection' => __('Book Sections','netblog'),
			'journal'=> __('Journal Articles','netblog'),
			'magazine' => __('Magazine Articles','netblog'),
			'newspaper' => __('Newspaper Articles','netblog'),
			'encyclopedia' => __('Encyclopedia Articles','netblog'),
			'conference' => __('Conference Proceedings','netblog'),
			'report' => __('Report','netblog'),
			'thesis' => __('Thesis','netblog'),
			'eric' => __('ERIC Document','netblog'),
			'website' => __('Websites','netblog'),
			'wiki' => __('Wiki','netblog'),
			'blog' => __('Blog','netblog'),
			'video' => __('Internet Video','netblog'),
			'powerpoint' => __('PowerPoint Presentation','netblog'),
			'art' => __('Art','netblog'),
			'recording' => __('Sound Recording','netblog'),
			'performance' => __('Performance','netblog'),
			'film' => __('Film','netblog'),
			'patent' => __('Patent','netblog'),
			'standard' => __('Standard (ISOs)','netblog'),
			'map' => __('Map','netblog'),
			'case' => __('Case','netblog')
		);
	}
	
	
	/**
	 * Convert a standard string into a parsable php-string, so that matched valid attributes may be parsed, such as 'author (year)' to '$author ($year)'.
	 *
	 * @uses ctype_alnum
	 * 
	 * @param string $cmd A string with attributes and without $
	 * @return string A safe php-parsable code to be used in eval()
	 */
	static public function parseCMD( $cmd )
	{
		$cmd = str_replace("$",'',$cmd);
		$cmd = strip_tags( stripslashes($cmd) ).' ';		
		
		// VALID VARS
		$dft = self::getDftAtts();
		$parsed = '';
		$hist = '';
		for( $i=0; $i<strlen($cmd); $i++ )
		{
			if( !ctype_alnum($char=$cmd[$i]) && $char != '_' ) {
				$parsed .= (strlen($hist)>0 && isset($dft[$hist]) ? "$$hist" : $hist) . $char;
				$hist = '';
			} else $hist .= $char;
		}
		$cmd = $parsed;	
		
		// VALID FORMAT
		$f = array( '-b-' => 'b', '-i-' => 'i', '-u-' => 'u', '-sub-'=>'sub', '-sup-'=>'-sup', '-c-'=>'lower', '-C-'=>'upper', '-Ca-' =>'cap' );
		foreach( $f as $code=>$tag ){
			$e = explode($code,' '.$cmd.' ');
			if( sizeof($e)%2 == 0 || sizeof($e) < 3 ) { $cmd = implode('',$e); continue; }
			
			for( $i=1; $i<sizeof($e); $i+=2 ) {
				if( $tag == 'lower' ) $e[$i] = "<span style=\"text-transform: lowercase;\">$e[$i]</span>";
				else if( $tag == 'upper' ) $e[$i] = "<span style=\"text-transform: uppercase;\">$e[$i]</span>";
				else if( $tag == 'cap' ) $e[$i] = "<span style=\"text-transform: capitalize;\">$e[$i]</span>";
				else if( $tag == 'u' ) $e[$i] = "<span style=\"text-decoration:underline\">$e[$i]</span>";
				else $e[$i] = "<$tag>$e[$i]</$tag>";
			}
			$cmd = implode('', $e); 
		}
		
		return addslashes($cmd);		
	}
	

	/**
	 * Checks if a name is a custom citation style.
	 *
	 * @param string $name The name which might be a custom citation style
	 * @return boolean TRUE if name is a valid custom citation style, FALSE otherwise
	 */
	static public function isCustomStyle( $name )
	{
		$s = explode( ',', self::getStyles() );
		return in_array($name,$s);		
	}

	
	/**
	 * Checks if a name is a default citation style.
	 *
	 * @param string $name The name which might be a default citation style
	 * @return boolean TRUE if name is a valid default citation style, FALSE otherwise
	 */
	static public function isDftStyle( $name )
	{
		$d = self::getDftStyles();
		return isset($d[strtolower($name)]);
	}
	
	
//---------------------------------------------------------------------------------------------------------------------
// HELPER FUNCTIONS
//---------------------------------------------------------------------------------------------------------------------	
	
	/**
	 * Generates a unique filter name, used to save in db.
	 *
	 * @param string $style A style name
	 * @param string $type A type name
	 * @return string Unique filter name
	 */
	static private function mkFilterName( $style, $type ) 
	{
		return "nbcite-$style-$type";
	}
	
	
	/**
	 * Extract style name from filter name
	 *
	 * @param string $fiter_name A filter name, as generated by mkFilterName()
	 * @return string The extracted style name
	 */
	static public function extractStyle( $fiter_name )
	{
		$t = explode('-',$fiter_name);
		return $t[1];
	}
	
	
	/**
	 * Extract type from filter name
	 *
	 * @param string $fiter_name A filter name, as generated by mkFilterName()
	 * @return string The extracted type name
	 */
	static public function extractType( $fiter_name )
	{
		$t = explode('-',$fiter_name);
		return $t[2];
	}	
	
	
	
	
	
	
	
}

//echo nbcstyle::parseCMD('publisher "title" -i-author-i- author bookauthor title_periodica   bookauthor');

?>